<?php
/**
 * Plugin Name: Supr3.me Security
 * Plugin URI:  http://www.supr3.me/plugin/supr3me-security/
 * Description: Supr3.me Security tightens the standard WordPress security with login activity logs & Google reCAPTCHA v2 on the login form
 * Version:     1.51
 * Author:      Supr3.me Plugins
 * Author URI:  http://www.supr3.me
 *
 * @author Supr3me Plugins <tech@supr3.me>
 */

// load in the Supr3meBaseAbstract class
if(!class_exists('Supr3meBaseAbstract'))
    require(__DIR__ . '/lib/Supr3meBaseAbstract.php');

// create Supr3meSecurity class
class Supr3meSecurity extends Supr3meBaseAbstract {

    public
        $table_name = null,
        $multisite  = false;


    /**
     * RedwingBaseAbstract Plugin constructor class
     * @param string $optionPrefix Prefix for plugin settings saved to database
     * @param array  $options      Array of permittable options for this plugin
     */
    public function __construct () {

        // get the database global
        global $wpdb;

        // run parent constructor with our options
        parent::__construct('security', array (
            // array of options along with default values below
            'display_captcha'           => 0,
            'hide_after_success'        => 1,
            'recaptcha_key_public'      => null,
            'recaptcha_key_private'     => null,
            'recaptcha_error_message'   => 'Incorrect reCAPTCHA, please try again.',
        ));

        // determine if we are on a multisite or not
        $this->multisite = (defined('MULTISITE') && (MULTISITE == true));

        // setup our custom storage table
        $this->table_name = ($wpdb->base_prefix ? $wpdb->base_prefix : $wpdb->prefix) . 'supr3me_security_log';

        // register plugin activation hook
        register_activation_hook(__FILE__ , array($this, 'register_activation_hook'));

        // run action after the plugins have loaded
        $this->action('plugins_loaded', 'setup_plugin');
    }

    /**
     * Method to run when the plugins have been loaded
     * @return void
     */
    public function setup_plugin () {

        // enqueue scripts & styles for the admin
        $this->action('admin_enqueue_scripts', function () {

            // supr3me security stylesheet
            $this->enqueue_style('supr3me-security.css');

            // supr3me security javascript
            $this->enqueue_script('supr3me-security.js', array('jquery'), true);

        });

        // setup the admin menu (network or normal)
        $this->action(($this->multisite ? 'network_' : null) . 'admin_menu', function () {

            // create the initial menu item (activity log)
            add_menu_page( 'Supr3me Security', 'Site Security', 'manage_options', 'supr3me-security', '__return_false', 'dashicons-lock', '107.513');

            // initial page to output the activity log
            add_submenu_page( 'supr3me-security', 'Activity Log', 'Activity Log', 'manage_options', 'supr3me-security', function () {

                // load initial activity log view
                $this->load_view('page_activity_log');

            });

            // add settings menu page
            add_submenu_page( 'supr3me-security', 'Supr3me Security Settings', 'Settings', 'manage_options', 'supr3me-security-settings', function () {

                // load settings page
                $this->load_view('page_settings');

            });

        });

        // handle admin post requests
        $this->action('admin_post_save_supr3me_settings', function () {

            // verify the submitted nonce
            if(!wp_verify_nonce($_REQUEST['__nonce'], '__nonce')) die('Invalid request');

            $tmp = array();

            // loop through each field and update the option
            foreach($_REQUEST['field'] as $key => $value) {

                // make sure this field is one of our options
                if(!in_array($key, array_keys($this->_options))) continue;

                // update the option and continue
                $this->update_option($key, $value);
            }

            // set updated notice
            $this->notice_set('Thanks, your settings have been updated.', 'updated');

            // redirect and exit
            wp_redirect(wp_get_referer()); exit;

        });


        // modify the login page
        $this->modify_login_page();

    }

    /**
     * Methods to modify the functionality of the login page
     * @return void
     */
    public function modify_login_page () {

        // enqueue scripts & styles for login page
        $this->action('login_enqueue_scripts', function () {

            // supr3me security stylesheet
            $this->enqueue_style('supr3me-security.css', array('dashicons'));

        });

        // add the reCAPTCHA to the middle of the form
        $this->action('login_form', function () {

            // check if we should be displaying the captcha
            if(!$this->show_captcha()) return;

            // output the reCAPTCHA into the login form
            $this->load_view('partial_recaptcha');

        });

        // check the reCAPTCHA code when the user logs in (make it a low priority)
        $this->action('authenticate', function ($user, $username, $password) {

            // by default, set $captcha_correct to false
            $captcha_correct = false;

            // check to ensure the $username and $password fields are populated
            if(strlen($username) < 1 && strlen($password) < 1) return $user;

            // first thing, lets check that captcha!
            if($this->show_captcha()) {

                // get our response from google
                $oResponse = $this->verifyCaptchaResponse($_POST['g-recaptcha-response']);

                // check to see if the user failed captcha
                if(!$oResponse->is_valid) {

                    // invalid response, change $user to WP_Error
                    return new WP_Error('authentication_failed', '<strong>ERROR</strong>: ' . esc_html($this->get_option('recaptcha_error_message')));
                }
            }


            // if we get to this point, the user (in theory) is a human
            // if the user is authenticated, drop a cookie
            if($user instanceof WP_User) {
                // OK the user has authenticated, drop our cookie
                setcookie('supr3me_recpvfy', md5($_SERVER['REMOTE_ADDR'] . AUTH_SALT), time() + ((3600 * 24) * 365), COOKIEPATH, COOKIE_DOMAIN);
            }

            // record our login attempt (could still be failed at this point)
            $this->record_login_attempt(($user instanceof WP_User), $username, $password);

            // return our WP_Error or WP_User object
            return $user;

        }, 200, 3);

    }

    /**
     * Method to verify our captcha response
     * @param string $key user captcha key
     * @param null|string $sIpAddress IP address or null to auto determine
     * @return StdClass
     */
    protected function verifyCaptchaResponse($key, $sIpAddress = null) {

        // determine our IP address
        if(is_null($sIpAddress)) {
            $sIpAddress = $_SERVER['REMOTE_ADDR'];
        }

        // create response object
        $oReturn = new StdClass();
        $oReturn->is_valid = false;
        $oReturn->error = 'unknown';

        // create cURL instance
        $oCurl = curl_init();

        // set our cURL params
        curl_setopt_array($oCurl, [
            CURLOPT_URL => 'https://www.google.com/recaptcha/api/siteverify',
            CURLOPT_TIMEOUT_MS => 3000,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_SSL_VERIFYHOST => false,
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_POST => true,
            CURLOPT_HTTPHEADER => [
                'Expects: application/json',
            ],
            CURLOPT_POSTFIELDS => [
                'secret' => $this->get_option('recaptcha_key_private'),
                'response' => $key,
                'remoteip' => $_SERVER['REMOTE_ADDR'],
            ]
        ]);

        // perform our query and get response
        $sResponse = curl_exec($oCurl);
        $oResponse = @json_decode($sResponse);

        // check we got our json back...
        if(is_object($oResponse)) {
            $oReturn->is_valid = $oResponse->success;
            $oReturn->error = property_exists('error-codes', $oResponse) ? $oResponse->{'error-codes'}[0] : false;
        }

        // return our response object
        return $oReturn;
    }


    /**
     * Method to record a login attempt on the activity database
     * @param  boolean $login_success  Whether the login was a success or not
     * @param  string  $username Username used to login with
     * @param  string  $password Password used to login with
     * @return void
     */
    public function record_login_attempt($login_success = true, $username, $password) {

        // get $wpdb so we can insert
        global $wpdb;

        // load our Crypter
        require_once($this->plugin_dir . '/lib/Supr3meCrypter.php');

        // insert the record
        $wpdb->insert($this->table_name, array (
            'loginsuccess'      => (int)$login_success,
            'blogid'            => get_current_blog_id(),
            'username'          => (string)$username,
            'password'          => ($login_success ? '' : Crypter::encrypt((string)$password)),
            'ipaddress'         => (string)$_SERVER['REMOTE_ADDR'],
            'hostname'          => gethostbyaddr($_SERVER['REMOTE_ADDR']),
            'useragent'         => (string)$_SERVER['HTTP_USER_AGENT'],
            'createdts'         => current_time('mysql'),
        ));

    }

    /**
     * Determines if the captcha should be shown or hidden
     * @return bool True if the captcha should be shown
     */
    public function show_captcha () {

        // check whether display captcha is set
        if(!$this->get_option('display_captcha')) return false;

        // check if the user has a valid cookie
        if($this->get_option('hide_after_success') && isset($_COOKIE['supr3me_recpvfy']) && $_COOKIE['supr3me_recpvfy'] == md5($_SERVER['REMOTE_ADDR'] . AUTH_SALT)) return false;

        // if we get to this point, return true
        return true;
    }

    /**
     * When registering the plugin, setup a custom table to store activity in
     * @return void
     */
    public function register_activation_hook () {

        // create global instance of $wpdb
        global $wpdb;

        // build up our $charset_collate string
        $charset_collate = null;
        if(!empty($wpdb->charset)) $charset_collate .= sprintf('DEFAULT CHARACTER SET %s', $wpdb->charset);
        if(!empty($wpdb->collate)) $charset_collate .= sprintf(' COLLATE %s', $wpdb->collate);

        // create our SQL
        $sql = sprintf(
            'CREATE TABLE IF NOT EXISTS %s (' .
                'id int(10) NOT NULL AUTO_INCREMENT, ' .
                'loginsuccess tinyint(1) DEFAULT \'1\' NOT NULL, ' .
                'blogid int(10) DEFAULT \'0\' NOT NULL, ' .
                'username varchar(255) DEFAULT \'\' NOT NULL, ' .
                'password varchar(255) DEFAULT \'\' NOT NULL, ' .
                'ipaddress varchar(20) DEFAULT \'\' NOT NULL, ' .
                'hostname varchar(255) DEFAULT \'\' NOT NULL, ' .
                'useragent varchar(255) DEFAULT \'\' NOT NULL, ' .
                'createdts datetime DEFAULT \'0000-00-00 00:00:00\' NOT NULL, ' .
                'UNIQUE KEY id (id)' .
            ') %s', $this->table_name, $charset_collate
        );

        // run the plugin table setup
        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
        dbDelta( $sql );
    }
}

// create new instance of the Vimeo upload
new Supr3meSecurity();